@@ -14,7 +14,7 @@
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
- Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
+ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
  USA
 **/
 
@@ -58,6 +58,9 @@ void KigMode::leftMouseMoved( QMouseEvent*, KigWidget* )
 
 void KigMode::leftReleased( QMouseEvent*, KigWidget* )
 {
+  /* insist disabling the undo button to avoid crashes */
+  mdoc.action( "edit_undo" )->setEnabled( false );
+  mdoc.action( "edit_redo" )->setEnabled( false );
 }
 
 void KigMode::midClicked( QMouseEvent*, KigWidget* )
