@@ -19,7 +19,7 @@
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
 
@@ -136,78 +136,101 @@ double XParser::a2fkt( Ufkt *u_item, double x, double h )
 
 void XParser::findFunctionName(QString &function_name, int const id, int const type)
 {
-        char last_character;
-        int pos;
-	if ( type == XParser::Polar || type == XParser::ParametricX || type == XParser::ParametricY)
-                pos=1;
-        else
-                pos=0;
-        for ( ; ; ++pos)
-        {
-                last_character = 'f';
-                for (bool ok=true; last_character<'x'; ++last_character)
-                {
-                        if ( pos==0 && last_character == 'r') continue;
-                        function_name.at(pos)=last_character;
-                        for( QValueVector<Ufkt>::iterator it = ufkt.begin(); it != ufkt.end(); ++it)
-                        {
-                                if (it == ufkt.begin() && it->fname.isEmpty() ) continue;
-                                if ( it->fstr.startsWith(function_name+'(') && (int)it->id!=id) //check if the name is free
-                                                ok = false;
-                        }
-                        if ( ok) //a free name was found
-                        {
-                                //kdDebug() << "function_name:" << function_name << endl;
-                                return;
-                        }
-                        ok = true;
-	       }
-               function_name.at(pos)='f';
-               function_name.append('f');
-        }
-        function_name = "e"; //this should never happen
+  char last_character;
+  int pos;
+  if ( function_name.length()==2/*type == XParser::Polar*/ || type == XParser::ParametricX || type == XParser::ParametricY)
+    pos=1;
+  else
+    pos=0;
+  for ( ; ; ++pos)
+  {
+    last_character = 'f';
+    for (bool ok=true; last_character<'x'; ++last_character)
+    {
+      if ( pos==0 && last_character == 'r') continue;
+      function_name.at(pos)=last_character;
+      for( QValueVector<Ufkt>::iterator it = ufkt.begin(); it != ufkt.end(); ++it)
+      {
+        if (it == ufkt.begin() && it->fname.isEmpty() ) continue;
+        if ( it->fstr.startsWith(function_name+'(') && (int)it->id!=id) //check if the name is free
+          ok = false;
+      }
+      if ( ok) //a free name was found
+      {
+        //kdDebug() << "function_name:" << function_name << endl;
+        return;
+      }
+      ok = true;
+    }
+    function_name.at(pos)='f';
+    function_name.append('f');
+  }
+  function_name = "e"; //this should never happen
 }
 
 void XParser::fixFunctionName( QString &str, int const type, int const id)
 {
-	int const p1=str.find('(');
-	int const p2=str.find(')');
-	if( p1>0 && str.at(p2+1)=='=')
-	{
-		QString const fname = str.left(p1);
-		for ( QValueVector<Ufkt>::iterator it = ufkt.begin(); it!=ufkt.end(); ++it )
-			if (it->fname == fname)
-			{
-				str = str.mid(p1,str.length()-1);
-				QString function_name;
-				if ( type == XParser::Polar )
-				  function_name = "rf";
-				else if ( type == XParser::ParametricX )
-				  function_name = "x";
-				else if ( type == XParser::ParametricY )
-				  function_name = "y";
-				else
-					function_name = "f";
-				findFunctionName(function_name, id, type);
-				str.prepend( function_name );
-				return;
-			}
-	}
-	else if ( p1==-1 || !str.at(p1+1).isLetter() ||  p2==-1 || str.at(p2+1 )!= '=')
-	{
-                QString function_name;
-                if ( type == XParser::Polar )
-                        function_name = "rf";
-		else if ( type == XParser::ParametricX )
-		  	function_name = "xf";
-		else if ( type == XParser::ParametricY )
-		  	function_name = "yf";
-                else
-                        function_name = "f";
-                str.prepend("(x)=");
-                findFunctionName(function_name, id, type);
-		str.prepend( function_name );
-	}
+  if ( str.startsWith( "y=" ) ) //we find a free function name
+  {
+    str.remove( 0, 2 );
+    str.prepend("(x)=");
+    QString function_name;
+    findFunctionName(function_name, id, type);
+    str.prepend( function_name );
+  }
+
+  int p1=str.find('(');
+  int p2=str.find(')');
+  if( p1>=0 && str.at(p2+1)=='=')
+  {
+    if ( type == XParser::Polar && str.at(0)!='r' )
+    {
+      if (str.at(0)=='(')
+      {
+        str.prepend('f');
+        p1++;
+        p2++;
+      }
+      str.prepend('r');
+      p1++;
+      p2++;
+    }
+    QString const fname = str.left(p1);
+    for ( QValueVector<Ufkt>::iterator it = ufkt.begin(); it!=ufkt.end(); ++it )
+    {
+      if (it->fname == fname)
+      {
+        str = str.mid(p1,str.length()-1);
+        QString function_name;
+        if ( type == XParser::Polar )
+          function_name = "rf";
+        else if ( type == XParser::ParametricX )
+          function_name = "x";
+        else if ( type == XParser::ParametricY )
+          function_name = "y";
+        else
+          function_name = "f";
+        findFunctionName(function_name, id, type);
+        str.prepend( function_name );
+        return;
+      }
+    }
+  }
+  else if ( p1==-1 || !str.at(p1+1).isLetter() ||  p2==-1 || str.at(p2+1 )!= '=')
+  {
+    QString function_name;
+    if ( type == XParser::Polar )
+      function_name = "rf";
+    else if ( type == XParser::ParametricX )
+      function_name = "xf";
+    else if ( type == XParser::ParametricY )
+      function_name = "yf";
+    else
+      function_name = "f";
+    str.prepend("(x)=");
+    findFunctionName(function_name, id, type);
+    str.prepend( function_name );
+  }
 }
 
 double XParser::euler_method(const double x, const QValueVector<Ufkt>::iterator it)
@@ -268,10 +291,8 @@ void XParser::prepareAddingFunction(Ufkt *temp)
         temp->f2_mode = false;
         temp->integral_mode = false;
         temp->integral_precision = Settings::stepWidth();
-        temp->dmin = 0;
-        temp->dmax = 0;
-        temp->str_dmin = "";
-        temp->str_dmax = "";
+        temp->usecustomxmin = false;
+	temp->usecustomxmax = false;
         temp->use_slider = -1;
         //TODO temp->slider_min = 0; temp->slider_max = 50;
 
@@ -643,7 +664,7 @@ int XParser::addFunction(const QString &f_str)
 	return id;
 }
 
-bool XParser::addFunction(const QString &fstr_const, bool f_mode, bool f1_mode, bool f2_mode, bool integral_mode, bool integral_use_precision, int linewidth, int f1_linewidth, int f2_linewidth, int integral_linewidth, const QString &str_dmin, const QString &str_dmax, const QString &str_startx, const QString &str_starty, double integral_precision, QRgb color, QRgb f1_color, QRgb f2_color, QRgb integral_color, QStringList str_parameter, bool use_slider)
+bool XParser::addFunction(const QString &fstr_const, bool f_mode, bool f1_mode, bool f2_mode, bool integral_mode, bool integral_use_precision, int linewidth, int f1_linewidth, int f2_linewidth, int integral_linewidth, const QString &str_dmin, const QString &str_dmax, const QString &str_startx, const QString &str_starty, double integral_precision, QRgb color, QRgb f1_color, QRgb f2_color, QRgb integral_color, QStringList str_parameter, int use_slider)
 {
 	QString fstr(fstr_const);
 	switch ( fstr.at(0).latin1() )
@@ -651,7 +672,6 @@ bool XParser::addFunction(const QString &fstr_const, bool f_mode, bool f1_mode,
 	  case 'r':
 	  {
 	    fixFunctionName(fstr, XParser::Polar);
-	    kdDebug() << "Hit?" << endl;
 	    break;
 	  }
 	  case 'x':
@@ -677,10 +697,23 @@ bool XParser::addFunction(const QString &fstr_const, bool f_mode, bool f1_mode,
 	added_function->f1_linewidth = f1_linewidth;
 	added_function->f2_linewidth = f2_linewidth;
 	added_function->integral_linewidth = integral_linewidth;
-	added_function->str_dmin = str_dmin;
-	added_function->str_dmax = str_dmax;
-	added_function->dmin = eval(str_dmin);
-	added_function->dmax = eval(str_dmax);
+	
+  if ( str_dmin.isEmpty() )
+    added_function->usecustomxmin = false;
+  else //custom minimum range
+  {
+    added_function->usecustomxmin = true;
+    added_function->str_dmin = str_dmin;
+    added_function->dmin = eval(str_dmin);
+  }
+  if ( str_dmax.isEmpty() )
+    added_function->usecustomxmax = false;
+  else //custom maximum range
+  {
+    added_function->usecustomxmax = true;
+    added_function->str_dmax = str_dmax;
+    added_function->dmax = eval(str_dmax);
+  }
 	added_function->str_startx = str_startx;
 	added_function->str_starty = str_starty;
 	if ( !str_starty.isEmpty() )
@@ -738,7 +771,7 @@ bool XParser::sendFunction(int id, const QString &dcopclient_target)
 	}
 	
 	Ufkt *item = &ufkt[ixValue(id)];
-	kdDebug() << "Snder " << item->fname.latin1() << endl;
+	kdDebug() << "Transferring " << item->fname.latin1() << endl;
 	QString str_result;
 	if ( dcopclient_target.isEmpty() && item->fname.at(0) == 'y' )
 	  	return false;
@@ -754,14 +787,21 @@ bool XParser::sendFunction(int id, const QString &dcopclient_target)
 	
 	QByteArray parameters;
 	QDataStream arg( parameters, IO_WriteOnly);
-
+ 
+  QString str_dmin;
+  if (!item->usecustomxmin)
+    str_dmin = item->str_dmin;
+  QString str_dmax;
+  if (!item->usecustomxmax)
+    str_dmax = item->str_dmax;
+  
 	QStringList str_parameters;
 	for ( QValueList<ParameterValueItem>::Iterator it = item->parameters.begin(); it != item->parameters.end(); ++it )
 	  	str_parameters.append( (*it).expression);
-	arg << item->fstr << item->f_mode << item->f1_mode << item->f2_mode << item->integral_mode << item->integral_use_precision << item->linewidth << item->f1_linewidth << item->f2_linewidth << item->integral_linewidth << item->str_dmin << item->str_dmax << item->str_startx << item->str_starty << item->integral_precision << item->color << item->f1_color << item->f2_color << item->integral_color << str_parameters << item->use_slider;
+	arg << item->fstr << item->f_mode << item->f1_mode << item->f2_mode << item->integral_mode << item->integral_use_precision << item->linewidth << item->f1_linewidth << item->f2_linewidth << item->integral_linewidth << str_dmin << str_dmax << item->str_startx << item->str_starty << item->integral_precision << item->color << item->f1_color << item->f2_color << item->integral_color << str_parameters << item->use_slider;
 	QByteArray replay_data;
 	QCString replay_type;
-	bool ok = kapp->dcopClient()->call( str_result.utf8(), "Parser", "addFunction(QString,bool,bool,bool,bool,bool,int,int,int,int,QString,QString,QString,QString,double,QRgb,QRgb,QRgb,QRgb,QStringList,bool)", parameters, replay_type, replay_data, false);
+	bool ok = kapp->dcopClient()->call( str_result.utf8(), "Parser", "addFunction(QString,bool,bool,bool,bool,bool,int,int,int,int,QString,QString,QString,QString,double,QRgb,QRgb,QRgb,QRgb,QStringList,int)", parameters, replay_type, replay_data, false);
 	if (!ok)
 	{
 		KMessageBox::error(0, i18n("An error appeared during the transfer"));
