@@ -12,9 +12,14 @@
 
 #ifndef KTOUCHUTILS_H
 #define KTOUCHUTILS_H
- 
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <qstring.h>
- 
+
+/// Formats the speed (given in chars/min) and creates a nice looking text.
 QString format_speed(double speed);
 
 #endif  // KTOUCHUTILS_H
